// SPDX-FileCopyrightText: 2025 Moritz Hanke <hanke@dkrz.de>
//
// SPDX-License-Identifier: BSD-3-Clause

#include <stdlib.h>
#include <stdio.h>
#include <mpi.h>

#include "elmer_coupling.h"
#include "elmer_icon_coupling.h"

int main (int argc, char *argv[]) {

  if (argc != 4) {
    fprintf(
      stderr,
      "wrong number of arguments\n"
      "usage: %s grid_dir num_parts config_file\n"
      "  - grid_dir: directory containing partitioned grid data\n"
      "  - num_parts: number of partitions in grid_dir\n"
      "  - config_file: path to configuration file\n", argv[0]);
    exit(EXIT_FAILURE);
  }
  char const * grid_dir = argv[1];
  int num_parts = atoi(argv[2]);
  char const * coupling_config_file = argv[3];

  MPI_Comm elmer_comm;

  // initialise coupler
  // has to be called as early as possible during the initialisation
  coupling_init(coupling_config_file, &elmer_comm);

  int comm_rank, comm_size;
  MPI_Comm_rank(elmer_comm, &comm_rank);
  MPI_Comm_size(elmer_comm, &comm_size);


  // initialise Elmer/Ice
  // ...

  // setup coupling with other components
  coupling_setup(grid_dir, num_parts);

  // time stepping loop
  int const nstep = 12;
  for (int step = 0; step < nstep; ++step) {

    if (comm_rank == 0) fprintf(stderr, "timestep: %d\n", step);

    // compute timestep
    {
      //....
    }

    // couple with ICON
    elmer_icon_coupling(comm_rank);
  }

  MPI_Comm_free(&elmer_comm);

  // finalise coupler
  coupling_finalize();

  return EXIT_SUCCESS;
}
