// SPDX-FileCopyrightText: 2025 Moritz Hanke <hanke@dkrz.de>
//
// SPDX-License-Identifier: BSD-3-Clause

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <mpi.h>
#include <netcdf.h>

#include "yac_core.h"
#include "icon_varlist.h"

void get_icon_varlist(
  char const * varlist_filename, char const * data_filename,
  struct icon_varlist ** varlist) {

  FILE * varlist_file = fopen(varlist_filename, "r");

  if (!varlist_file) {
    fputs("could not open varlist files\n", stderr);
    MPI_Abort(MPI_COMM_WORLD, EXIT_FAILURE);
  }

  size_t num_vars = 0;
  char const ** varnames = NULL;
  char line[128] = {'\0'};
  while (fgets(line, 128, varlist_file) != NULL) {
    char varname[64];
    sscanf(line, "%s", varname);
    varnames = realloc(varnames, (num_vars + 1) * sizeof(*varnames));
    varnames[num_vars] = strdup(varname);
    ++num_vars;
  }

  fclose(varlist_file);

  *varlist = malloc(sizeof(**varlist) + num_vars * sizeof((*varlist)->vars[0]));
  (*varlist)->num_vars = num_vars;

  yac_nc_open(data_filename, 0, &((*varlist)->ncid));

  int ncells_dim_id;
  yac_nc_inq_dimid((*varlist)->ncid, "ncells", &ncells_dim_id);
  YAC_HANDLE_ERROR(
    nc_inq_dimlen((*varlist)->ncid, ncells_dim_id, &((*varlist)->ncells)));

  for (size_t i = 0; i < num_vars; ++i) {

    (*varlist)->vars[i].name = strdup(varnames[i]);
    yac_nc_inq_varid(
      (*varlist)->ncid, varnames[i], &((*varlist)->vars[i].varid));
    char var_attribute[128];
    memset(var_attribute, '\0', sizeof(var_attribute));
    YAC_HANDLE_ERROR(
      nc_get_att_text(
        (*varlist)->ncid, (*varlist)->vars[i].varid, "long_name",
        var_attribute));
    (*varlist)->vars[i].long_name = strdup(var_attribute);
    memset(var_attribute, '\0', sizeof(var_attribute));
    YAC_HANDLE_ERROR(
      nc_get_att_text(
        (*varlist)->ncid, (*varlist)->vars[i].varid, "units",
        var_attribute));
    (*varlist)->vars[i].units = strdup(var_attribute);
  }

  free(varnames);
}

void free_icon_varlist(struct icon_varlist * varlist) {

  for (size_t i = 0; i < varlist->num_vars; ++i) {
    free((void*)varlist->vars[i].name);
    free((void*)varlist->vars[i].long_name);
    free((void*)varlist->vars[i].units);
  }
  YAC_HANDLE_ERROR(nc_close(varlist->ncid));
  free(varlist);
}
